package cd.casic.ci.common.pipeline.pojo.element.trigger;

import cd.casic.ci.common.pipeline.enums.StartType;
import cd.casic.ci.common.pipeline.enums.TriggerRepositoryType;
import cd.casic.ci.common.pipeline.pojo.element.ElementProp;
import cd.casic.ci.common.pipeline.pojo.element.trigger.enums.PathFilterType;
import cd.casic.ci.common.pipeline.utils.TriggerElementPropUtils;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.experimental.Accessors;

import java.util.List;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.element.trigger
 * @Project：ops-pro
 * @name：CodeSVNWebHookTriggerElement
 * @Date：2025/03/27 10:09
 * @Filename：CodeSVNWebHookTriggerElement
 * @description：Todo
 */
@Accessors(chain = true)
@Schema(title = "SVN仓库代码提交触发", description = CodeSVNWebHookTriggerElement.classType)
public class CodeSVNWebHookTriggerElement extends WebHookTriggerElement {
    public static final String classType = "codeSVNWebHookTrigger";

    @Schema(title = "任务名称", required = true)
    private String name = "SVN事件触发";

    @Schema(title = "id", required = false)
    private String id;

    @Schema(title = "状态", required = false)
    private String status;

    @Schema(title = "仓库ID", required = true)
    private String repositoryHashId;

    @Schema(title = "路径过滤类型", required = true)
    private PathFilterType pathFilterType = PathFilterType.NamePrefixFilter;

    @Schema(title = "相对路径", required = true)
    private String relativePath;

    @Schema(title = "排除的路径", required = false)
    private String excludePaths;

    @Schema(title = "用户黑名单", required = false)
    private List<String> excludeUsers;

    @Schema(title = "用户白名单", required = false)
    private List<String> includeUsers;

    @Schema(title = "新版的svn原子的类型")
    private TriggerRepositoryType repositoryType;

    @Schema(title = "新版的svn代码库名")
    private String repositoryName;

    public CodeSVNWebHookTriggerElement(String name, String id, String status) {
        super(name, id, status);
    }

    @Override
    public String getClassType() {
        return classType;
    }

    @Override
    public String findFirstTaskIdByStartType(StartType startType) {
        return StartType.WEB_HOOK.name().equals(startType.name()) ? this.id : super.findFirstTaskIdByStartType(startType);
    }

    @Override
    public List<ElementProp> triggerCondition() {
        return List.of(
                TriggerElementPropUtils.vuexInput("relativePath", relativePath),
                TriggerElementPropUtils.vuexInput("excludePaths", excludePaths),
                TriggerElementPropUtils.staffInput("includeUsers", includeUsers),
                TriggerElementPropUtils.staffInput("excludeUsers", excludeUsers)
        ).stream().filter(java.util.Objects::nonNull).toList();
    }
}
