package cd.casic.ci.process.process.service.stage;

import cd.casic.ci.common.pipeline.req.stage.StageCreateReq;
import cd.casic.ci.common.pipeline.req.stage.StageReq;
import cd.casic.ci.common.pipeline.req.stage.StageUpdateReq;
import cd.casic.ci.common.pipeline.resp.stage.StageResp;
import cd.casic.ci.process.process.dataObject.stage.PipStage;
import com.baomidou.mybatisplus.extension.service.IService;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import jakarta.validation.constraints.NotNull;

import java.util.List;

public interface StageService extends IService<PipStage> {
    /**
     * 创建阶段及关联任务
     * @param stage 阶段信息
     * @return 阶段id
     */
    String createStagesOrTask(StageCreateReq stage);


    /**
     * 查询所有阶段任务
     * @param pipelineId 流水线id
     * @return 任务
     */
    List<StageResp> findAllStagesTask(String pipelineId);

    /**
     * 删除阶段及任务
     * @param taskId 配置id
     */
    void deleteStagesOrTask(String taskId);

    /**
     *  删除流水线所有阶段
     * @param pipelineId 流水线id
     */
    void  deleteAllStagesOrTask(String pipelineId);


    /**
     * 获取所有阶段的根节点
     * @param pipelineId 流水线id
     * @return 主分支
     */
    List<PipStage> findAllMainStage(String pipelineId);

    /**
     * 根据根节点查询从节点
     * @param stagesId 根节点id
     * @return 从节点列表
     */
    List<PipStage> findSecondStage(String stagesId);

    /**
     * 更新阶段任务
     * @param stage 更新内容
     */
    void updateStagesTask(StageUpdateReq stage);


    /**
     * 效验阶段配置必填字段
     * @param pipelineId 流水线id
     * @return 配置id集合
     */
    List<String> validStagesMustField(String pipelineId);

    /**
     * 更新阶段
     * @param stage 阶段信息
     */
    void updateStages(PipStage stage);

    /**
     * 删除阶段
     * @param stageId 阶段id
     */
    void deleteStages(String stageId);


    void copyStage(@NotEmpty String stageId);

}
