package cd.casic.pipeline.store.api.atom;

import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.pipeline.store.pojo.atom.AtomPipeline;
import cd.casic.pipeline.store.pojo.atom.AtomPipelineExecInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

@Tag(name = "USER_MARKET_ATOM_STATISTIC", description = "插件市场-插件-统计")
@RestController
@RequestMapping("/user/market/atom/statistic")
public interface UserMarketAtomStatisticResource {

    @Operation(summary = "根据插件代码获取使用的流水线详情 - 所有")
    @GetMapping("/{atomCode}/pipelines")
    PageResult<AtomPipeline> getAtomPipelinesByCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "插件代码", required = true)
            @PathVariable("atomCode")
            String atomCode,
            @Parameter(description = "第几页", required = false, example = "1")
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页多少条", required = false, example = "20")
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "根据插件代码获取对应的流水线信息 - 项目下")
    @GetMapping("/projectCodes/{projectCode}/atomCodes/{atomCode}/pipelines")
    PageResult<AtomPipelineExecInfo> getAtomPipelines(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目代码", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "原子代码", required = true)
            @PathVariable("atomCode")
            String atomCode,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );
}