package cd.casic.pipeline.store.api.container;

import cd.casic.ci.common.pipeline.pojo.OS;
import cd.casic.ci.common.pipeline.type.BuildType;
import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.container.ContainerResource;
import cd.casic.pipeline.store.pojo.container.ContainerResp;
import cd.casic.pipeline.store.pojo.container.ContainerType;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;
import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID_DEFAULT_VALUE;

@Tag(name = "USER_PIPELINE_CONTAINER", description = "流水线-构建容器")
@RestController
@RequestMapping("/user/pipeline/container")
public interface UserContainerResource {

    @Operation(summary = "获取构建容器资源信息")
    @GetMapping("/projects/{projectCode}/containers/{containerId}/oss/{os}")
    CommonResult<ContainerResource> getContainerResource(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目编码", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "容器ID", required = true)
            @PathVariable("containerId")
            String containerId,
            @Parameter(description = "操作系统", required = true)
            @PathVariable("os")
            OS os,
            @Parameter(description = "资源类型", required = false)
            @RequestParam("buildType")
            BuildType buildType
    );

    @Operation(summary = "获取构建容器资源信息")
    @GetMapping("/projects/{projectCode}/oss/{os}")
    CommonResult<ContainerResource> getContainerResource(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目编码", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "操作系统", required = true)
            @PathVariable("os")
            OS os,
            @Parameter(description = "资源类型", required = false)
            @RequestParam("buildType")
            BuildType buildType
    );

    @Operation(summary = "获取所有的流水线构建容器信息")
    @GetMapping("/{projectCode}")
    CommonResult<List<ContainerResp>> getAllContainerInfos(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目编码", required = true)
            @PathVariable("projectCode")
            String projectCode
    );

    @Operation(summary = "根据容器类型获取流水线构建容器信息")
    @GetMapping("/{projectCode}/{type}")
    CommonResult<List<ContainerResp>> getContainerInfoByType(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目编码", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "容器类型(trigger:触发器 vmBuild:构建环境 normal:无编译环境)", required = true)
            @PathVariable("type")
            String type
    );

    @Operation(summary = "根据容器类型和操作系统获取流水线构建容器信息")
    @GetMapping("/{projectCode}/{type}/{os}")
    CommonResult<List<ContainerResp>> getContainerInfoByTypeAndOs(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目编码", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "容器类型(trigger:触发器 vmBuild:构建环境 normal:无编译环境)", required = true)
            @PathVariable("type")
            String type,
            @Parameter(description = "操作系统", required = true)
            @PathVariable("os")
            OS os
    );

    @Operation(summary = "获取全部流水线构建容器信息")
    @GetMapping("/all")
    CommonResult<List<ContainerType>> getAllContainers();
}