package cd.casic.server.controller;

import cd.casic.ci.common.pipeline.req.pipeline.PipelineCreateReq;
import cd.casic.ci.common.pipeline.req.pipeline.PipelineQueryReq;
import cd.casic.ci.common.pipeline.req.pipeline.PipelineUpdateReq;
import cd.casic.ci.common.pipeline.resp.pipeline.PipelineFindResp;
import cd.casic.ci.process.process.service.pipeline.PipelineService;
import cd.casic.framework.commons.pojo.CommonResult;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import jakarta.annotation.Resource;
import jakarta.annotation.security.PermitAll;
import jakarta.validation.Valid;
import org.jetbrains.annotations.NotNull;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @ClassName PipelineController
 * @Author hopeli
 * @Date 2025/5/10 10:28
 * @Version 1.0
 */

@RestController
@RequestMapping("/pipeline")
public class PipelineController {

    @Resource
    private PipelineService pipelineService;

    @PermitAll
    @PostMapping(path="/createPipeline")
    public CommonResult<String> createPipeline(@RequestBody @Valid PipelineCreateReq pipelineReq){

        String pipelineId = pipelineService.createPipeline(pipelineReq);

        return CommonResult.success(pipelineId);
    }


    @PermitAll
    @PostMapping(path="/findPipelineList")
    public CommonResult<List<PipelineFindResp>> findPipelineList(@RequestBody @Valid PipelineQueryReq pipelineQueryReq){

        List<PipelineFindResp> respList = pipelineService.findPipelineList(pipelineQueryReq);

        return CommonResult.success(respList);
    }

    @PostMapping(path="/deletePipeline")
    public CommonResult<Void> deletePipeline(@NotNull String pipelineId){

        pipelineService.deletePipeline(pipelineId);

        return CommonResult.success();
    }


    @PostMapping(path="/updatePipeline")
    public CommonResult<Void> updatePipeline(@RequestBody @NotNull @Valid PipelineUpdateReq pipeline){

        pipelineService.updatePipeline(pipeline);

        return CommonResult.success();
    }


    @PostMapping(path="/findPipelinePage")
    public CommonResult<Page<PipelineFindResp>> findPipelinePage(@RequestBody @NotNull @Valid PipelineQueryReq query){

        Page<PipelineFindResp> pipelineRespPage = pipelineService.findPipelinePage(query);

        return CommonResult.success(pipelineRespPage);
    }




    @PostMapping(path="/pipelineClone")
    public CommonResult<Void> pipelineClone(@NotNull String pipelineId,@NotNull String pipelineName){

        pipelineService.pipelineClone(pipelineId,pipelineName);

        return CommonResult.success();
    }
}
